import argparse
from catalogclient import catalog
from datetime import datetime
import logging
from logging.config import fileConfig
import os
import re


class Sanitizer(object):

    def sanitize_products(self, product_type, start_date, end_date, virt_root):

        logger = logging.getLogger()

        cat = catalog.Catalog()
        products = cat.get_products(product_type, startdate=start_date, enddate=end_date)

        logger.info('found %i products to bioparsanitizer', len(products))

        product_root = os.path.join(virt_root, product_type)
        if not os.path.exists(product_root):
            os.mkdir(product_root)
            logger.debug('created %s', product_root)
        else:
            logger.debug('%s already existed', product_root)

        for product in products:
            try:
                self.symlink_product(product_root, product)
            except:
                logger.error('product %s could not be sanitized', self.get_filename(product))

    def symlink_product(self, product_root, product):

        logger = logging.getLogger()
        logger.debug('bioparsanitizer product %s', product.files[0])

        year = product.timestamp.year
        year_root = os.path.join(product_root, str(year))
        if not os.path.exists(year_root):
            os.mkdir(year_root)
            logger.info('created %s', year_root)
        else:
            logger.debug('%s already existed', year_root)

        date_root = os.path.join(year_root, str(product.timestamp.strftime('%Y%m%d')))
        if not os.path.exists(date_root):
            os.mkdir(date_root)
            logger.info('created %s', date_root)
        else:
            logger.debug('%s already existed', date_root)
        product_source = os.path.join(date_root, self.get_productsource(product))
        product_target = os.path.abspath(self.get_filename(product))
        # no symlink found
        if not os.path.islink(product_source):
            logger.info('create symlink %s to %s', product_source, product_target)
            os.symlink(product_target, product_source)
        # symlink found, but updated since previous run
        elif os.readlink(product_source) != product_target:
            logger.info('change symlink %s to %s', product_source, product_target)
            os.remove(product_source)
            os.symlink(product_target, product_source)
        # symlink found and correct
        else:
            logger.debug('symlink %s already existed', product_source)

    def get_productsource(self, product):

        filename = product.files[0].filename
        index = filename.rindex('/')+1
        rt_strip = re.sub(r'-RT.', '', str(filename[index:]))
        return rt_strip

    def get_filename(self, product):

        filename = product.files[0].filename
        index = filename.index('/')
        return filename[index:]


_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data(path):
    return os.path.join(_ROOT, path)


def parse_arguments():

    parser = argparse.ArgumentParser(prog='bioparsanitizer.py')

    parser.add_argument("--productType",type=str,help="Product type",
                        required=True)
    parser.add_argument("--startDate",type=str,help="Start date formatted as YYYY-MM-DD",
                        required=False,default='1990-01-01')
    parser.add_argument("--endDate",type=str,help="End date formatted as YYYY-MM-DD",
                        required=False,default='2020-01-01')
    parser.add_argument("--virtDirRoot",type=str,help="Virtual directory root",
                        required=True)

    return parser.parse_args()


if __name__ == '__main__':

    args = parse_arguments()

    fileConfig(get_data('logging.conf'))

    start_date = datetime.strptime(args.startDate, '%Y-%m-%d')
    end_date = datetime.strptime(args.endDate, '%Y-%m-%d')

    sanitizer = Sanitizer()
    sanitizer.sanitize_products(args.productType, start_date, end_date, args.virtDirRoot)